# -*- coding: utf-8 -*-
# $Id$

module MakeProj
  ##
  #	プロジェクト
  #
  class Project < Property
    #
    def initialize(name, property = nil)
      super(property)
      @name = name
      @configurations = Hash.new { |hash, key|
        hash[key] = Configuration.new(key, self)
      }
      @type = nil
      @tools_version = '4.0'
      @xmlns = 'http://schemas.microsoft.com/developer/msbuild/2003'
      # @guid = UUIDTools::UUID.timestamp_create.to_s
      @guid = nil
      @keyword = nil
      @pre_props = [ '$(VCTargetsPath)\Microsoft.Cpp.Default.props' ]
      @props = [ '$(VCTargetsPath)\Microsoft.Cpp.props' ]
      @post_props = [ '$(VCTargetsPath)\Microsoft.Cpp.targets' ]
      @sources = Array.new
      @headers = Array.new
      @character_set = 'Unicode'
      setup
    end

    # ソースファイルの配列
    attr_reader :sources

    # ヘッダファイルの配列
    attr_reader :headers

    # 初期化
    def setup
    end

    #
    # 構成を追加する
    #
    def add_configuration(configuration, platform = 'Win32')
      str = "#{configuration}|#{platform}"
      if @configurations.include?(str)
        raise(Error, "duplicate configuration, '#{str}'")
      end
      return @configurations[str]
    end

    #
    def make
      doc = REXML::Document.new
      root = doc.add_element('Project', 
                             'DefaultTargets' => 'Build', 
                             'ToolsVersion' => @tools_version, 
                             'xmlns' => @xmlns)

      item = root.add_element('ItemGroup', 
                              'Label' => 'ProjectConfigurations')
      @configurations.each_value { |configuration|
        elem = item.add_element('ProjectConfiguration', 
                                'Include' => configuration.to_s)
        elem.add_element('Configuration').add_text(configuration.name)
        elem.add_element('Platform').add_text(configuration.platform)
      }

      item = root.add_element('PropertyGroup', 
                              'Label' => 'Globals')
      if @guid
        item.add_element('ProjectGuid').add_text("{#{@guid}}")
      end
      if @keyword
        item.add_element('Keyword').add_text(@keyword)
      end
      # item.add_element('RootNamespace').add_text(@name)

      add_props(root, @pre_props)

      @configurations.each_value { |config|
        group = root.add_element('PropertyGroup', 
                                 'Condition' => config.condition, 
                                 'Label' => 'Configuration')
        group.add_element('ConfigurationType').add_text(@type)
        group.add_element('UseDebugLibraries').add_text(config.is_debug.to_s)
        if @character_set
          group.add_element('CharacterSet').add_text(@character_set)
        end
      }

      add_props(root, @props)

      @configurations.each_value { |config|
        group = root.add_element('ImportGroup', 
                                 'Label' => 'PropertySheets', 
                                 'Condition' => config.condition)
        config.props.each { |prop|
          group.add_element('Import', 
                            'Project' => prop, 
                            'Condition' => "exists('#{prop}')", 
                            'Label' => 'LocalAppDataPlatform')
        }

        property_group = root.add_element('PropertyGroup', 
                                          'Condition' => config.condition)
        config.properties.add_xml(property_group)

        definition = root.add_element('ItemDefinitionGroup', 
                                      'Condition' => config.condition)
        compile = definition.add_element('ClCompile')
        config.compile_options.add_xml(compile)
        lib = definition.add_element('Lib')
        config.lib_options.add_xml(lib)
        link = definition.add_element('Link')
        config.link_options.add_xml(link)
      }
      group = root.add_element('ItemGroup')
      headers.each { |header|
        group.add_element('ClInclude', 'Include' => header)
      }
      sources.each { |source|
        group.add_element('ClCompile', 'Include' => source)
      }

      add_props(root, @post_props)

      return doc
    end

    #
    # プロジェクトファイルを作成する。
    #
    def create(filename = nil)
      filename ||= "#{@name}.vcxproj"
      xml = make
      File.open(filename, 'w') { |fd|
        xml.write(fd, 2, true)
        # xml.write(fd)
      }
    end

    #
    # .filtersファイルを作成する
    #
    def create_filter(filename = nil)
      filename ||= "#{@name}.vcxproj.filters"
      doc = REXML::Document.new
      root = doc.add_element('Project', 
                             'ToolsVersion' => @tools_version, 
                             'xmlns' => @xmlns)
      filters = Array.new
      (@headers + @sources).each { |path|
        filter = get_filter(path)
        while !filters.include?(filter)
          filters << filter.dup
          if !filter.gsub!(/\\.+?$/, '')
            break
          end
        end
      }

      item_group = root.add_element('ItemGroup')
      filters.each { |filter|
        item_group.add_element('Filter', 'Include' => filter)
      }
      
      add_filter_xml(root, @headers, 'ClInclude')
      add_filter_xml(root, @sources, 'ClCompile')

      File.open(filename, 'w') { |fd|
        doc.write(fd, 2, true)
        # doc.write(fd)
      }
    end

    private

    #
    # プロパティシートを追加する
    #
    def add_props(elem, props)
      props.each { |prop|
        elem.add_element('Import', 'Project' => prop)
      }
    end

    #
    # パスからフィルタ名を求める
    #
    def get_filter(path)
      return File.dirname(path).tr("/", "\\")
    end

    #
    # パスからフィルタを書き込む
    #
    def add_filter_xml(element, files, type)
      item_group = element.add_element('ItemGroup')
      files.each { |path|
        item_group.add_element(type, 'Include' => path).
        add_element('Filter').add_text(get_filter(path))
      }
    end
  end

  ##
  #  Win32ベースプロジェクト
  #
  class Win32Project < Project
    #
    def initialize(name)
      super(name)
      @keyword = 'Win32Proj'
      append_macro('WIN32')
      append_macro('%(PreprocessorDefinitions)')
    end
  end

  ##
  #  静的ライブラリプロジェクト
  #
  class StaticLibraryProject < Win32Project
    #
    def initialize(name)
      super(name)
      @type = 'StaticLibrary'
      append_macro('_LIB')
    end
  end

  ##
  #  コンソールアプリケーション
  #
  class ConsoleApplicationProject < Win32Project
    #
    def initialize(name)
      super(name)
      @type = 'Application'
      append_macro('_CONSOLE')
      set_subsystem('Console')
    end
  end
end
